/****************************************************************************
   Copyright & License:
   ====================
   
   Copyright 2009 - 2010 gAGE/UPC & ESA
   
   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at
   
   http://www.apache.org/licenses/LICENSE-2.0
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 ***************************************************************************/

/****************************************************************************
 * Copyright: gAGE/UPC & ESA
 * Project: EDUNAV GNSS Lab Tool 
 * Author: Pere Ramos-Bosch (group of Astronomy and GEomatics - gAGE/UPC)
 * Maintainer: Adria Rovira-Garcia ( gAGE/UPC )
 *             glab.gage @ upc.edu           
 * File: input.h
 * Code Management Tool File Version: 2.0  Revision: 0 
 * Date: 2010/11/30
 ***************************************************************************/

/****************************************************************************
 * MODULE DESCRIPTION
 *
 * Name: input
 * Language: C
 *
 * Purpose:
 *  The purpose of this module is to serve as "driver" from the different ways
 *  that the data can be stored in the form of files and the rest of the program
 *
 * Function:
 *  This class is in charge of reading the different data formats in which the 
 *  data can be stored, and save it to the internal data structures.
 *
 * Dependencies:
 *  dataHandling
 *
 * Files modified:
 *  None
 *
 * Files read:
 *  Depending on function
 *
 * Resources usage:
 *  See Design Document
 ****************************************************************************/

/****************************
 *       RELEASE_HISTORY
 * -------------
 *          gLAB v1.3.1
 * Release: 2010/01/22
 * Change Log: First public version.
 * -------------
 *          gLAB v1.3.2
 * Release: 2010/02/23
 * Change Log: Included input files: SINEX, DCB and GPS_Receiver_Type.
 *             Forward/Backward processing included.
 * -------------
 *          gLAB v1.4.0
 * Release: 2010/06/21
 * Change Log: No changes in this file.
 * -------------
 *          gLAB v1.4.1
 * Release: 2010/07/09
 * Change Log: No changes in this file.
 * -------------
 *          gLAB v1.4.2
 * Release: 2010/07/31
 * Change Log: No changes in this file.
 * -------------
 *          gLAB v1.4.3
 * Release: 2010/08/31
 * Change Log: No changes in this file.
 * -------------
 *          gLAB v1.4.4
 * Release: 2010/09/22
 * Change Log: No changes in this file.
 * -------------
 *          gLAB v1.4.5
 * Release: 2010/10/31
 * Change Log: No changes in this file.
 * -----------
 *          gLAB v2.0.0
 * Release: 2010/11/30
 * Change Log: gLAB version released together with the Educational Book:
 *             "Global Navigation Satellite Systems: Fundamentals, Algorithms and Laboratory Exercises"
 *             Authors: J. Sanz, JM. Juan and M. Hernandez-Pajares.
 * -----------
 *       END_RELEASE_HISTORY
 *****************************/

#ifndef INPUT_H_
#define INPUT_H_

/* System modules */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>

/* External classes */
#include "dataHandling.h"
#include "output.h"

/* Internal types, constants and data */
#define		BACKWARD_READ_BATCH		150

// Input functions
int getL(char *lineptr, int *n, FILE *stream);
int getLback(char *lineptr, int *n, FILE *stream);

// File management
enum fileType whatFileTypeIs (char *filename);

// RINEX Observation
int readRinexObsHeader (FILE *fd, TEpoch *epoch, TOptions *options);
int readRinexObsEpoch (FILE *fd, TEpoch *epoch,TConstellation *constellation, enum ProcessingDirection direction);
int rewindEpochRinexObs (FILE *fd, enum Source src);
void beforeReadEpochUpdate(TEpoch *epoch);
void afterReadEpochUpdate(TEpoch *epoch,TConstellation *constellation);

// RINEX Navigation message
int readRinexNav (FILE *fd, TGNSSproducts *products);
int readRinexNavFile (char *filename, TGNSSproducts *products);

// SP3 files
int readSP3 (FILE *fd, TGNSSproducts *products, int readOrbits, int readClocks);
int readSP3File (char *filename, TGNSSproducts *products, int readOrbits, int readClocks);

// RINEX Clocks
int readRinexClocks (FILE *fd, TGNSSproducts *products);
int readRinexClocksFile (char *filename, TGNSSproducts *products);

// IONEX
int readIONEX (FILE *fd, TIONEX *IONEX);
int readIONEXFile (char *filename, TIONEX *IONEX);

// Satellite constellation
int readConstellation (FILE *fd, TConstellation *constellation);
int readConstellationFile (char *filename, TConstellation *constellation);

// ANTEX
int readAntex (FILE *fd, TConstellation *constellation, TAntennaList *antennaList);
int readAntexFile (char *filename, TConstellation *constellation, TAntennaList *antennaList);

// DCB
int readDCB (FILE *fd, TDCBdata *DCB);
int readDCBFile (char *filename, TDCBdata *DCB);

// GPS Receiver types
int readRecType (FILE *fd, TReceiverList *recList);
int readRecTypeFile (char *filename, TReceiverList *recList);

// SINEX
int readSINEX (FILE *fd, TStationList *stationList);
int readSINEXFile (char *filename, TStationList *stationList);

#endif /*INPUT_H_*/
