/****************************************************************************
   Copyright & License:
   ====================
   
   Copyright 2009 - 2010 gAGE/UPC & ESA
   
   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at
   
   http://www.apache.org/licenses/LICENSE-2.0
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 ***************************************************************************/

/****************************************************************************
 * Copyright: gAGE/UPC & ESA
 * Project: EDUNAV GNSS Lab Tool 
 * Author: Pere Ramos-Bosch (group of Astronomy and GEomatics - gAGE/UPC)
 * Maintainer: Adria Rovira-Garcia ( gAGE/UPC )
 *             glab.gage @ upc.edu     
 * File: output.h
 * Code Management Tool File Version: 2.0  Revision: 0 
 * Date: 2010/11/30
 ***************************************************************************/

/****************************************************************************
 * MODULE DESCRIPTION
 *
 * Name: output
 * Language: C
 *
 * Purpose:
 *  The purpose of this module is to act as output for the solution of the filter
 *
 * Function:
 *  This class is in charge of outputting the results of the Kalman filter done 
 *  by the filter module
 *
 * Dependencies:
 *  dataHandling, filter, model
 *
 * Files modified:
 *  None
 *
 * Files read:
 *  None
 *
 * Resources usage:
 *  See Design Document
 ****************************************************************************/

/****************************
 *       RELEASE_HISTORY
 * -------------
 *          gLAB v1.3.1
 * Release: 2010/01/22
 * Change Log: First public version.
 * -------------
 *          gLAB v1.3.2
 * Release: 2010/02/23
 * Change Log: Imported SATDIFF, SATSTAT and SATSTATTOT printing from gLAB.c
 *             Added satellite SATPVT message.
 *             Change in SATDIFF to show the GNSS system that the satellite belongs to.
 *             PREFIT message now includes the weight that will be applied to the filter.
 *             ZTD and estimated residual tropospheric delay is now printed in the FILTER message .
 *             SATSTAT message will now provide the GNSS system it refers to.
 * -------------
 *          gLAB v1.4.0
 * Release: 2010/06/21
 * Change Log: No changes in this file.
 * -------------
 *          gLAB v1.4.1
 * Release: 2010/07/09
 * Change Log: No changes in this file.
 * -------------
 *          gLAB v1.4.2
 * Release: 2010/07/31
 * Change Log: No changes in this file.
 * -------------
 *          gLAB v1.4.3
 * Release: 2010/08/31
 * Change Log: Satellite elevation and azimuth addition in the print MODEL message.
 * -------------
 *          gLAB v1.4.4
 * Release: 2010/09/22
 * Change Log: No changes in this file.
 * -------------
 *          gLAB v1.4.5
 * Release: 2010/10/31
 * Change Log: No changes in this file.
 * -----------
 *          gLAB v2.0.0
 * Release: 2010/11/30
 * Change Log: gLAB version released together with the Educational Book:
 *             "Global Navigation Satellite Systems: Fundamentals, Algorithms and Laboratory Exercises"
 *             Authors: J. Sanz, JM. Juan and M. Hernandez-Pajares.
 * -----------
 *       END_RELEASE_HISTORY
 *****************************/

#ifndef OUTPUT_H_
#define OUTPUT_H_

/* System modules */
#include <stdio.h>

/* External classes */
#include "dataHandling.h"
#include "filter.h"
#include "model.h"

/* Internal types, constants and data */

void printParameters(TOptions *options, TEpoch *epoch, TGNSSproducts *products);
void printError (char *message, TOptions *options);
void printInfo (char *message, TOptions *options);
void printInput(TEpoch *epoch, TOptions *options);
 void printMeas(TEpoch *epoch, TOptions *options);
void printCS (TEpoch *epoch, int satIndex, int LiCheck, double LiLiestDiff, double LiThreshold, int BwCheck, double nBwdiff, double nBwdiffThreshold, int L1C1Check, double L1C1diff, double L1C1Threshold, TOptions *options);
void printModel (TEpoch *epoch, int satIndex, int measIndex, double flightTime, double geometricDistance, double satelliteClockModel, double satellitePhaseCenterProjection, double receiverPhaseCenterProjection, double receiverARPProjection, double relativityModel, double windUpMetModel, double troposphereModel, double ionosphereModel, double gravitationalDelayModel, double TGDModel, double solidTidesModel, TOptions *options);
void printSatSel (int selected, int satellite, char *message, TOptions *options);
void printPrefit (TTime *t, TSatellite *sat, enum MeasurementType, double prefit, double measurement, double model, double weight, TOptions *options);
void printEpochsat (TTime *t, enum MeasurementType measType, int ns, int *sats, TOptions *options);
void printFilter (TTime *t, int nunk, TFilterSolution *solution, TOptions *options);
void printPostfit (TTime *t, TSatellite *sat, enum MeasurementType measType, double postfit, double measurement, double model, double estimatedParameters, double ambiguity, TOptions *options);
void printOutput (TEpoch *epoch, TFilterSolution *solution, TOptions *options, TUnkinfo *unkinfo);
void printSatDiff (TTime *t, enum GNSSystem GNSS, int PRN, double SISREDiff, double OrbDiff, double Orb3DDiff, double ClkDiff, double posDiff[3], TOptions *options);
void printSatStat (enum GNSSystem GNSS, int PRN, TStat statSISRE, TStat statOrb, TStat stat3D, TStat statClk, TStat statRadOrb, TStat statAloOrb, TStat statCroOrb, TOptions *options);
void printSatStatTot (TStat statSISRETot, TStat statOrbTot, TStat stat3DTot, TStat statClkTot, TStat statRadOrbTot, TStat statAloOrbTot, TStat statCroOrbTot, TOptions *options);
void printSatPvt (TTime *t, enum GNSSystem GNSS, int PRN, double position[3], double velocity[3], double clock, TOptions *options);

#endif /*OUTPUT_H_*/
